import json
from collections import OrderedDict
import os


def _prompt(prompt):
    """
    Prompt user for input.
    """
    return str(input(prompt))


def yes_or_no(question):
    while "the answer is invalid":
        reply = _prompt(question + " (y/n): ").lower().strip()
        if reply[:1] == "y":
            return True
        if reply[:1] == "n":
            return False


def get_annotation_type():
    while "the answer is invalid":

        q = (
            "What type of annotation is this? Possible types are...\n"
            + "  multiselect -- checkboxes where users can pick 0 or more\n"
            + "  radio -- radio buttons where users must pick 1\n"
            + "  text -- free text entry box\n"
            + "  likert -- a likert scale with an order list of radio buttons\n"
            + "  bws -- a set of options where users select the most/least extreme "
            + "options w.r.t. some scale\n\n"
        )

        options = ["multiselect", "radio", "text", "likert", "btw"]

        reply = _prompt(q).lower().strip()
        if reply in options:
            return reply


def get_initial_config():
    config = OrderedDict()

    config["server_name"] = "potato annotator"

    config["user_config"]: {"allow_all_users": True, "users": []}

    config["alert_time_each_instance"] = 10000000

    # The html that changes the visualiztation for your task. Change this file
    # to influence the layout and description of your task. This is not a full
    # HTML page, just the piece that does lays out your task's pieces
    config["html_layout"] = "templates/examples/plain_layout.html"

    # The core UI files for Potato. You should not need to change these normally.
    #
    # Exceptions to this might include:
    # 1) You want to add custom CSS/fonts to style your task
    # 2) Your layout requires additional JS/assets to render
    # 3) You want to support additional keybinding magic
    #
    config["base_html_template"] = "templates/base_template.html"
    config["header_file"] = "templates/header.html"

    # This is where the actual HTML files will be generated
    config["site_dir"] = "potato/templates/"

    return config


def create_task_cli():

    print("Welcome to the Potato annotation task creation process on the command line!")
    print("This process will walk you through a series of questions about your task")
    print("and generate a .yaml file that has the initial configuration for your")
    print("task. You can edit this .yaml file at any time after the process to tweak")
    print("the intructions, add fancier features, or even change the nature of the")
    print("annotation. This process will only take a few minutes.")
    print("")
    print("If you don't yet know the answer to any of these questions you can leave")
    print("the answer blank and edit your .yaml file later to fill in these details.")

    print("")
    print("First, some preliminary questions about how you want the server setup:")

    config = get_initial_config()

    config["annotation_task_name"] = _prompt(
        "What is the name for your annotation task" + " that will be shown to users?\n"
    )

    while True:
        port = _prompt("What port do you want the server to run on? ")
        try:
            port = int(port)
            if port > 1 and port < 16000:
                config["port"] = port
                break
        except ValueError:
            print("%s needs to be a valid numeric port number")

    data_files = []
    fname = _prompt("What is the absolute path to one of your data files? ")
    data_files.append(fname)

    # Let the user entire more files
    while yes_or_no("Do you have more data files?"):
        fname = _prompt("What is the absolute path to another data files? ")
        data_files.append(fname)
    config["data_files"] = data_files

    id_key = _prompt("Which field/column in the data file is the item's ID? ")
    text_key = _prompt("Which field/column in the data file is the item's text? ")
    context_key = _prompt(
        "(optional) Which field/column in the data file is the item's additional context? "
    )

    ip = {"id_key": id_key, "text_key": text_key}
    if len(context_key) > 0:
        ip["contex_key"] = context_key

    config["item_properties"] = ip

    config["annotation_codebook_url"] = _prompt("What is the URL for the annotation codebook? ")

    config["output_annotation_dir"] = _prompt(
        "What is the absolute path for the directory "
        + "where the annotations should be written?\n"
    )

    config["output_annotation_format"] = _prompt(
        "What format do you want the annotations written in?\n"
        + "Options: csv, tsv, json, jsonl\n\n"
    )

    print("\nNow we're going to ask about the specific ways you want the data annotated")

    annotation_schemes = []
    while True:

        scheme = {}

        atype = get_annotation_type()

        desc = _prompt("What description/question/instructions should annotators see for this?\n")
        name = _prompt("What is the internal name for this category (used in output files)?\n")

        if atype == "likert" or atype == "bws":
            min_label = ""
            max_label = ""

            if atype == "likert":
                size = _prompt("How many items are on the likert scale?\n")
            else:
                size = _prompt("How many items to dislay at once for Best-Worst Scaling?\n")

            scheme["min_label"] = min_label
            scheme["max_label"] = max_label
            scheme["size"] = size

        elif atype == "text":
            pass

        elif atype == "mutliselect" or atype == "radio":
            # Get the options
            labels = []
            label = _prompt(
                "Enter the text for one option (or press enter with no input when done): "
            )
            while label != "":
                labels.append(label)
                label = _prompt(
                    "Enter the text for one option (or press enter with no input when done): "
                )

            scheme["labels"] = labels

        annotation_schemes.append(scheme)

        if not yes_or_no("Are there more annotation types/tasks to add?"):
            break

    config["annoation_schemes"] = annotation_schemes

    while True:
        config_file = _prompt(
            "What is the absolute path for where this config.yaml file should be written?\n"
        )

        if os.path.exists(config_file):
            if not yes_or_no("Config file already exists. Overwrite?"):
                continue

        with open(config_file, "wt") as f:
            json.dump(config, f, indent=4)
        break



"""
Likert Layout
"""

from .radio import generate_radio_layout


def generate_likert_layout(annotation_scheme):

    # If the user specified the more complicated likert layout, default to the
    # radio layout
    if "labels" in annotation_scheme:
        return generate_radio_layout(annotation_scheme, horizontal=False)

    for required in ["size", "min_label", "max_label"]:
        if required not in annotation_scheme:
            raise Exception(
                'Likert scale for "%s" did not include %s' % (annotation_scheme["name"], required)
            )

    schematic = (
        '<div><form action="/action_page.php">'
        + '  <fieldset> <legend>%s</legend> <ul class="likert" style="text-align: center;"> <li> %s </li>'
    ) % (annotation_scheme["description"], annotation_scheme["min_label"])

    key2label = {}
    label2key = {}
    key_bindings = []

    # Setting up label validation for each label, if "required" is True,
    # the annotators will be asked to finish the current instance to proceed
    validation = ""
    label_requirement = annotation_scheme.get("label_requirement")
    if label_requirement and label_requirement.get("required"):
        validation = "required"

    for i in range(1, annotation_scheme["size"] + 1):

        label = "scale_" + str(i)
        name = annotation_scheme["name"] + ":::" + label
        class_name = annotation_scheme["name"]

        key_value = str(i % 10)

        # if the user wants us to add in easy key bindings
        if "sequential_key_binding" in annotation_scheme and annotation_scheme["sequential_key_binding"] == True and annotation_scheme["size"] < 10:
            key2label[key_value] = label
            label2key[label] = key_value
            key_bindings.append((key_value, class_name + ": " + key_value))

        # In the collapsed version of the likert scale, no label is shown.
        label_content = str(i) if annotation_scheme.get("displaying_score") else ""
        tooltip = ""

        # displaying the label content in a different line if it is not empty
        if label_content != "":
            line_break = "<br>"
        else:
            line_break = ""
        # schematic += \
        #        ((' <li><input class="%s" type="radio" id="%s" name="%s" value="%s" onclick="onlyOne(this)">' +
        #         '  <label for="%s" %s>%s</label></li>')
        #         % (class_name, label, name, key_value, name, tooltip, label_content))

        schematic += (
            (
                ' <li><input class="{class_name}" type="radio" id="{id}" name="{name}" value="{value}" onclick="onlyOne(this)" validation="{validation}">'
                + ' {line_break} <label for="{label_for}" {label_args}>{label_text}</label></li>'
            )
        ).format(
            class_name=class_name,
            id=name,
            name=name,
            value=key_value,
            validation=validation,
            line_break=line_break,
            label_for=name,
            label_args=tooltip,
            label_text=" " + label_content,
        )

    # allow annotators to choose bad_text label
    bad_text_schematic = ""
    if "label_content" in annotation_scheme.get("bad_text_label", {}):
        name = annotation_scheme["name"] + ":::" + "bad_text"
        bad_text_schematic = (
            (
                ' <li><input class="{class_name}" type="radio" id="{id}" name="{name}" value="{value}" onclick="onlyOne(this)" validation="{validation}">'
                + ' {line_break} <label for="{label_for}" {label_args}>{label_text}</label></li>'
            )
        ).format(
            class_name=annotation_scheme["name"],
            id=name,
            name=name,
            value=0,
            validation=validation,
            line_break="<br>",
            label_for=name,
            label_args="",
            label_text=annotation_scheme["bad_text_label"]["label_content"],
        )
        if "sequential_key_binding" in annotation_scheme and annotation_scheme["sequential_key_binding"] == True and annotation_scheme["size"] < 10:
            key_bindings.append(
                (0, class_name + ": " + annotation_scheme["bad_text_label"]["label_content"])
            )

    schematic += "  <li>%s</li> %s </ul></fieldset>\n</form></div>\n" % (
        annotation_scheme["max_label"],
        bad_text_schematic,
    )

    return schematic, key_bindings



from argparse import ArgumentParser
import yaml
import os
import pandas as pd
import json
from collections import defaultdict


def arguments():
    """
    Creates and returns the arg parser for Potato on the command line
    """
    parser = ArgumentParser()
    parser.set_defaults(show_path=False, show_similarity=False)

    parser.add_argument("multitask_config_file")

    return parser.parse_args()


def main():
    args = arguments()

    # load multitask annotation config
    with open(args.multitask_config_file, "rt") as f:
        multitask_config = yaml.safe_load(f)

    # create basic folders
    for folder in ["surveyflow", "annotation_output", "data_files", "configs", "htmls"]:
        cur_path = multitask_config["base_dir"] + folder
        if not os.path.exists(cur_path):
            os.makedirs(cur_path)
            print("Created directory: %s" % (cur_path))

    # build mapping dict
    key2text = defaultdict(dict)

    if "multitask_guideline_file" in multitask_config and os.path.exists(
        multitask_config["multitask_guideline_file"]
    ):
        # load multitask annotation guideline
        multitask_guideline_df = pd.read_csv(multitask_config["multitask_guideline_file"])
        for i, row in multitask_guideline_df.iterrows():
            if type(row["key"]) != str or row["key"][0] != "[" or row["key"][-1] != "]":
                continue
            for task in multitask_config["tasks"]:
                key2text[row["key"]][task] = (
                    row[task] if type(row[task]) == str else row[multitask_config["base_task"]]
                )

    for task in multitask_config["tasks"]:
        # load basic config file
        with open(multitask_config["base_config_file"], "rt") as f:
            page = f.read()
            for key in key2text:
                page = page.replace(key, key2text[key][task])

            surveyflow_output_path = multitask_config["surveyflow_output_path"].replace(
                "[TASK]", task
            )
            page = page.replace(
                multitask_config["surveyflow_path"], surveyflow_output_path + task + "-"
            )
            page = page.replace("[TASK]", task)

            config = yaml.safe_load(page)

        config["output_annotation_dir"] = multitask_config["output_annotation_dir"].replace(
            "[TASK]", task
        )
        if not os.path.exists(config["output_annotation_dir"]):
            os.makedirs(config["output_annotation_dir"])

        """
        # setup the site_dir path for each task
        config['path_under_site_dir'] = multitask_config['path_under_site_dir'].replace("[TASK]", task)
        config["site_dir"] += config['path_under_site_dir']
        if not os.path.exists(config["site_dir"]):
            os.makedirs(config["site_dir"])
        """

        config["annotation_task_name"] = multitask_config["annotation_task_name"].replace(
            "[TASK]", task
        )
        config["data_files"] = [
            it.replace("[TASK]", task) for it in multitask_config["data_files"]
        ]
        # config["prestudy"] = [it.replace("[TASK]", task) for it in multitask_config['prestudy']]

        with open(multitask_config["base_dir"] + "configs/%s.yaml" % task, "wt") as f:
            json.dump(config, f, indent=4)

        # create the dir for surveyflow_output_path
        if not os.path.exists(surveyflow_output_path):
            os.makedirs(surveyflow_output_path)

        # setup surveyflow pages
        surveyflow_files = os.listdir(multitask_config["surveyflow_path"])
        for file in surveyflow_files:
            if os.path.isdir(multitask_config["surveyflow_path"] + file):
                continue
            with open(multitask_config["surveyflow_path"] + file, "r") as f:
                page = f.read()
            for key in key2text:
                page = page.replace(key, key2text[key][task])
            page = page.replace("[TASK]", task)
            # print(surveyflow_output_path)
            with open(surveyflow_output_path + task + "-" + file, "wt") as f:
                f.write(page)

        # for key in ["surveyflow_output_path", ]


if __name__ == "__main__":
    main()



"""
Span Layout
"""

import logging
from collections.abc import Mapping
from potato.server_utils.config_module import config

logger = logging.getLogger(__name__)


SPAN_COLOR_PALETTE = [
    "(230, 25, 75)",
    "(60, 180, 75)",
    "(255, 225, 25)",
    "(0, 130, 200)",
    "(245, 130, 48)",
    "(145, 30, 180)",
    "(70, 240, 240)",
    "(240, 50, 230)",
    "(210, 245, 60)",
    "(250, 190, 212)",
    "(0, 128, 128)",
    "(220, 190, 255)",
    "(170, 110, 40)",
    "(255, 250, 200)",
    "(128, 0, 0)",
    "(170, 255, 195)",
    "(128, 128, 0)",
    "(255, 215, 180)",
    "(0, 0, 128)",
    "(128, 128, 128)",
    "(255, 255, 255)",
    "(0, 0, 0)",
]


def get_span_color(span_label):
    """
    Returns the color of a span with this label as a string with an RGB triple
    in parentheses, or None if the span is unmapped.
    """
    if "ui" not in config or "spans" not in config["ui"]:
        return None
    span_ui = config["ui"]["spans"]

    if "span_colors" not in span_ui:
        return None

    if span_label in span_ui["span_colors"]:
        return span_ui["span_colors"][span_label]
    else:
        return None


def set_span_color(span_label, color):
    """
    Sets the color of a span with this label as a string with an RGB triple in parentheses.

    :color: a string containing an RGB triple in parentheses
    """
    if "ui" not in config:
        ui = {}
        config["ui"] = ui
    else:
        ui = config["ui"]

    if "spans" not in ui:
        span_ui = {}
        ui["spans"] = span_ui
    else:
        span_ui = ui["spans"]

    if "span_colors" not in span_ui:
        span_colors = {}
        span_ui["span_colors"] = span_colors
    else:
        span_colors = span_ui["span_colors"]

    span_colors[span_label] = color


def render_span_annotations(text, span_annotations):
    """    
    Retuns a modified version of the text with span annotation overlays inserted
    into the text.

    :text: some instance to be annotated
    :span_annotations: annotations already made by the user that need to be
       re-inserted into the text
    """
    # This code is synchronized with the javascript function
    # surroundSelection(selectionLabel) function in base_template.html which
    # wraps any labeled text with a <div> element indicating its label. We
    # replicate this code here (in python).
    #
    # This synchrony also means that any changes to the UI code for rendering
    # need to be updated here too.

    # We need to go in reverse order to make the string update in the right
    # places, so make sure things are ordered in reverse of start

    rev_order_sa = sorted(span_annotations, key=lambda d: d["start"], reverse=True)

    ann_wrapper = (
        '<span class="span_container" selection_label="{annotation}" '
        + 'style="background-color:rgb{bg_color};">'
        + "{span}"
        + '<div class="span_label" '
        + 'style="background-color:white;border:2px solid rgb{color};">'
        + "{annotation}</div></span>"
    )
    for a in rev_order_sa:

        # Spans are colored according to their order in the list and we need to
        # retrofit the color
        color = get_span_color(a["annotation"])
        # The color is an RGB triple like (1,2,3) and we want the background for
        # the text to be somewhat transparent so we switch to RGBA for bg
        bg_color = color.replace(")", ",0.25)")

        ann = ann_wrapper.format(
            annotation=a["annotation"], span=a["span"], color=color, bg_color=bg_color
        )
        text = text[: a["start"]] + ann + text[a["end"] :]

    return text


def generate_span_layout(annotation_scheme, horizontal=False):
    """
    Renders a span annotation option selection in the annotation panel and
    returns the HTML code
    """
    # when horizontal is specified in the annotation_scheme, set horizontal = True
    if "horizontal" in annotation_scheme and annotation_scheme["horizontal"]:
        horizontal = True

    schematic = (
        '<form action="/action_page.php">'
        + "  <fieldset>"
        + ("  <legend>%s</legend>" % annotation_scheme["description"])
    )

    # TODO: display keyboard shortcuts on the annotation page
    key2label = {}
    label2key = {}
    key_bindings = []

    # setting up label validation for each label, if "required" is True, the annotators will be asked to finish the current instance to proceed
    validation = ""
    label_requirement = (
        annotation_scheme["label_requirement"] if "label_requirement" in annotation_scheme else None
    )
    if label_requirement and ("required" in label_requirement) and label_requirement["required"]:
        validation = "required"

    for i, label_data in enumerate(annotation_scheme["labels"], 1):

        label = label_data if isinstance(label_data, str) else label_data["name"]

        name = annotation_scheme["name"] + ":::" + label
        class_name = annotation_scheme["name"]
        key_value = name

        span_color = get_span_color(label)
        if span_color is None:
            span_color = SPAN_COLOR_PALETTE[(i - 1) % len(SPAN_COLOR_PALETTE)]
            set_span_color(label, span_color)

        # For better or worse, we need to cache these label-color pairings
        # somewhere so that we can render them in the colored instances later in
        # render_span_annotations(). The config object seems like a reasonable
        # place to do since it's global and the colors are persistent
        config["ui"]

        tooltip = ""
        if isinstance(label_data, Mapping):
            tooltip_text = ""
            if "tooltip" in label_data:
                tooltip_text = label_data["tooltip"]
                # print('direct: ', tooltip_text)
            elif "tooltip_file" in label_data:
                with open(label_data["tooltip_file"], "rt") as f:
                    lines = f.readlines()
                tooltip_text = "".join(lines)
                # print('file: ', tooltip_text)
            if len(tooltip_text) > 0:
                tooltip = (
                    'data-toggle="tooltip" data-html="true" data-placement="top" title="%s"'
                    % tooltip_text
                )

            # Bind the keys
            if "key_value" in label_data:
                key_value = label_data["key_value"]
                if key_value in key2label:
                    logger.warning("Keyboard input conflict: %s" % key_value)
                    quit()
                key2label[key_value] = label
                label2key[label] = key_value
                key_bindings.append((key_value, class_name + ": " + label))
            # print(key_value)

        if (
            "sequential_key_binding" in annotation_scheme
            and annotation_scheme["sequential_key_binding"]
            and len(annotation_scheme["labels"]) <= 10
        ):
            key_value = str(i % 10)
            key2label[key_value] = label
            label2key[label] = key_value
            key_bindings.append((key_value, class_name + ": " + label))

        if "displaying_score" in annotation_scheme and annotation_scheme["displaying_score"]:
            label_content = label_data["key_value"] + "." + label
        else:
            label_content = label

        # Check the first radio
        if i == 1:
            is_checked = 'xchecked="checked"'
        else:
            is_checked = ""

        # TODO: add support for horizontal layout
        br_label = "<br/>"
        if horizontal:
            br_label = ""

        # We want to mark that this input isn't actually an annotation (unlike,
        # say, checkboxes) so we prefix the name with span_label so that the
        # answer ingestion code in update_annotation_state() can skip over which
        # radio was checked as being annotations that need saving (while the
        # spans themselves are saved)
        name_with_span = "span_label:::" + name

        schematic += (
            '      <input class="{class_name}" for_span="{for_span}" type="checkbox" id="{name}" name="{name_with_span}" '
            + ' value="{key_value}" {is_checked} '
            + "onclick=\"onlyOne(this); changeSpanLabel(this, '{label_content}', '{span_color}');\" validation=\"{validation}\">"
            + '  <label for="{name}" {tooltip}>'
            + '<span style="background-color:rgb{bg_color};">{label_content}</span></label>{br_label}'
        ).format(
            class_name=class_name,
            for_span = True,
            name=name,
            key_value=key_value,
            label_content=label_content,
            validation=validation,
            tooltip=tooltip,
            br_label=br_label,
            is_checked=is_checked,
            name_with_span=name_with_span,
            bg_color=span_color.replace(")", ",0.25)"),
            span_color=span_color,
        )

    # allow annotators to choose bad_text label
    bad_text_schematic = ""
    if "label_content" in annotation_scheme.get("bad_text_label", {}):
        name = annotation_scheme["name"] + ":::" + "bad_text"
        bad_text_schematic = (
            (
                    ' <input class="{class_name}" for_span="{for_span}" type="checkbox" id="{id}" name="{name}" value="{value}" onclick="onlyOne(this)" validation="{validation}">'
                    + ' {line_break} <label for="{label_for}" {label_args}>{label_text}</label>'
            )
        ).format(
            class_name=annotation_scheme["name"],
            for_span=True,
            id=name,
            name=name,
            value=0,
            validation=validation,
            line_break="",
            label_for=name,
            label_args="",
            label_text=annotation_scheme["bad_text_label"]["label_content"],
        )
        if (
                "sequential_key_binding" in annotation_scheme
                and annotation_scheme["sequential_key_binding"]
                and len(annotation_scheme["labels"]) <= 10
        ):
            key_bindings.append(
                (0, class_name + ": " + annotation_scheme["bad_text_label"]["label_content"])
            )

    schematic += bad_text_schematic

    schematic += "  </fieldset>\n</form>\n"
    return schematic, key_bindings



"""
Multiselect Layout
"""

import logging
from collections.abc import Mapping

logger = logging.getLogger(__name__)


def generate_multiselect_layout(annotation_scheme):
    schematic = (
        '<form action="/action_page.php">'
        + "  <fieldset>"
        + ("  <legend>%s</legend>" % annotation_scheme["description"])
    )

    # TODO: display keyboard shortcuts on the annotation page
    key2label = {}
    label2key = {}

    key_bindings = []

    display_info = (
        annotation_scheme["display_config"] if "display_config" in annotation_scheme else {}
    )

    n_columns = display_info["num_columns"] if "num_columns" in display_info else 1

    schematic += "<table>"

    # setting up label validation for each label, if "required" is True, the annotators will be asked to finish the current instance to proceed
    validation = ""
    label_requirement = (
        annotation_scheme["label_requirement"] if "label_requirement" in annotation_scheme else None
    )
    if label_requirement and label_requirement["required"]:
        validation = "required"

    # if right_label is provided, the associated label has to be clicked to proceed. This is normally used for consent questions at the beginning of a survey.
    right_label = set()
    if label_requirement and "right_label" in label_requirement:
        if type(label_requirement["right_label"]) == str:
            right_label.add(label_requirement["right_label"])
        elif type(label_requirement["right_label"]) == list:
            right_label = set(label_requirement["right_label"])
        else:
            logger.warning("Incorrect format of right_label %s" % label_requirement["right_label"])
            # quit()

    for i, label_data in enumerate(annotation_scheme["labels"], 1):

        if (i - 1) % n_columns == 0:
            schematic += "<tr>"
        schematic += "<td>"

        label = label_data if isinstance(label_data, str) else label_data["name"]

        name = annotation_scheme["name"] + ":::" + label
        class_name = annotation_scheme["name"]
        key_value = name

        tooltip = ""
        if isinstance(label_data, Mapping):
            tooltip_text = ""
            if "tooltip" in label_data:
                tooltip_text = label_data["tooltip"]
                # print('direct: ', tooltip_text)
            elif "tooltip_file" in label_data:
                with open(label_data["tooltip_file"], "rt") as f:
                    lines = f.readlines()
                tooltip_text = "".join(lines)
                # print('file: ', tooltip_text)
            if len(tooltip_text) > 0:
                tooltip = (
                    'data-toggle="tooltip" data-html="true" data-placement="top" title="%s"'
                    % tooltip_text
                )

            if "key_value" in label_data:
                key_value = label_data["key_value"]
                if key_value in key2label:
                    logger.warning("Keyboard input conflict: %s" % key_value)
                    quit()
                key2label[key_value] = label
                label2key[label] = key_value
                key_bindings.append((key_value, class_name + ": " + label))

        if (
            "sequential_key_binding" in annotation_scheme
            and annotation_scheme["sequential_key_binding"]
            and len(annotation_scheme["labels"]) <= 10
        ):
            key_value = str(i % 10)
            key2label[key_value] = label
            label2key[label] = key_value

        label_content = label
        if annotation_scheme.get("video_as_label", None) == "True":
            assert (
                "videopath" in label_data
            ), "Video path should in each label_data when video_as_label is True."
            video_path = label_data["videopath"]
            label_content = f"""
            <video width="320" height="240" autoplay loop muted>
                <source src="{video_path}" type="video/mp4" />
            </video>"""

        # add shortkey to the label so that the annotators will know how to use it
        # when the shortkey is "None", this will not displayed as we do not allow short key for None category
        # if label in label2key and label2key[label] != 'None':
        # if label in label2key:
        #    label_content = label_content + \
        #        ' [' + label2key[label].upper() + ']'

        final_validation = "right_label" if label in right_label else validation

        if ("single_select" in annotation_scheme) and (
            annotation_scheme["single_select"] == "True"
        ):
            logger.warning(
                'single_select is Depricated and will be removed soon. Use "radio" instead.'
            )
            schematic += (
                '  <input class="%s" type="checkbox" id="%s" name="%s" value="%s" onclick="onlyOne(this)" validation="%s">'
                + '  <label for="%s" %s>%s</label><br/>'
            ) % (class_name, name, name, key_value, final_validation, name, tooltip, label_content)
        else:
            schematic += (
                '<label for="%s" %s><input class="%s" type="checkbox" id="%s" name="%s" value="%s" onclick="whetherNone(this)" validation="%s">'
                + "  %s</label><br/>"
            ) % (name, tooltip, class_name, name, name, key_value, final_validation, label_content)

        schematic += "</td>"
        if i % n_columns == 0:
            schematic += "</tr>"

    if "has_free_response" in annotation_scheme and annotation_scheme["has_free_response"]:

        label = "free_response"
        name = annotation_scheme["name"] + ":::free_response"
        class_name = annotation_scheme["name"]
        tooltip = "Entire a label not listed here"
        instruction = (
            "Other"
            if "instruction" not in annotation_scheme["has_free_response"]
            else annotation_scheme["has_free_response"]["instruction"]
        )

        schematic += (
            '<tr><td colspan="%s"><div style="float:left; display:flex; flex-direction:row;">%s <input class="%s" type="text" id="%s" name="%s">'
            + '  <label for="%s" %s></label></div></td</tr>'
        ) % (str(n_columns), instruction, class_name, name, name, name, tooltip)

    schematic += "</table>"
    schematic += "  </fieldset>\n</form>\n"

    return schematic, key_bindings



from .likert import generate_likert_layout
from .multiselect import generate_multiselect_layout
from .multirate import generate_multirate_layout
from .number import generate_number_layout
from .pure_display import generate_pure_display_layout
from .radio import generate_radio_layout
from .select import generate_select_layout
from .span import generate_span_layout
from .textbox import generate_textbox_layout
